/*
 * Copyright (C) 2021-2022 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#include "cache_flush_tests_xehp.inl"
#include "get_device_info_xehp.inl"
#include "hw_helper_tests_xehp.inl"
#include "hw_info_tests_xehp.inl"
#include "memory_manager_tests_xehp.inl"
#include "sampler_tests_xehp.inl"
#include "test_buffer_xe_hp_sdv.inl"
#include "test_command_stream_receiver_xehp.inl"
#include "test_device_caps_xehp.inl"
#include "test_hw_info_config_xehp.inl"
#include "test_image_xe_hp_sdv.inl"
#include "test_local_work_size_xehp.inl"
#include "test_platform_caps_xehp.inl"
#include "test_sub_devices_xehp.inl"
