/*
 * Copyright (C) 2021 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#include "shared/source/helpers/compiler_hw_info_config.h"

namespace NEO {

CompilerHwInfoConfig *CompilerHwInfoConfigFactory[IGFX_MAX_PRODUCT] = {};

} // namespace NEO
