;
; Copyright (C) 2018-2021 Intel Corporation
;
; SPDX-License-Identifier: MIT
;

LIBRARY    "gdi32_mock"
EXPORTS
D3DKMTOpenAdapterFromHdc
D3DKMTOpenAdapterFromLuid
D3DKMTCreateAllocation
D3DKMTCreateAllocation2
D3DKMTDestroyAllocation
D3DKMTDestroyAllocation2
D3DKMTQueryAdapterInfo
D3DKMTCloseAdapter
D3DKMTCreateDevice
D3DKMTDestroyDevice
D3DKMTEscape
D3DKMTShareObjects
D3DKMTCreateContextVirtual
D3DKMTDestroyContext
D3DKMTOpenResource
D3DKMTOpenResourceFromNtHandle
D3DKMTQueryResourceInfo
D3DKMTQueryResourceInfoFromNtHandle
D3DKMTLock
D3DKMTUnlock
D3DKMTRender
D3DKMTCreateSynchronizationObject
D3DKMTCreateSynchronizationObject2
D3DKMTDestroySynchronizationObject
D3DKMTSignalSynchronizationObject
D3DKMTWaitForSynchronizationObject
D3DKMTWaitForSynchronizationObjectFromCpu
D3DKMTSignalSynchronizationObjectFromCpu
D3DKMTWaitForSynchronizationObjectFromGpu
D3DKMTSignalSynchronizationObjectFromGpu
D3DKMTCreatePagingQueue
D3DKMTDestroyPagingQueue
D3DKMTLock2
D3DKMTUnlock2
D3DKMTMapGpuVirtualAddress
D3DKMTReserveGpuVirtualAddress
D3DKMTFreeGpuVirtualAddress
D3DKMTUpdateGpuVirtualAddress
D3DKMTSubmitCommand
D3DKMTMakeResident
D3DKMTEvict
D3DKMTGetDeviceState
D3DKMTRegisterTrimNotification
D3DKMTUnregisterTrimNotification
D3DKMTCreateHwQueue
D3DKMTDestroyHwQueue
D3DKMTSubmitCommandToHwQueue
D3DKMTSetAllocationPriority
MockSetAdapterInfo
MockSetSizes
GetMockSizes
GetMockLastDestroyedResHandle
SetMockLastDestroyedResHandle
GetMockCreateDeviceParams
SetMockCreateDeviceParams
getMockAllocation
getAdapterInfoAddress
getLastCallMapGpuVaArg
getLastCallReserveGpuVaArg
setMapGpuVaFailConfig
getCreateContextData
getCreateHwQueueData
getDestroyHwQueueData
getSubmitCommandToHwQueueData
getDestroySynchronizationObjectData
getMonitorFenceCpuFenceAddress
getCreateSynchronizationObject2FailCall
getRegisterTrimNotificationFailCall
getLastPriority
setAdapterBDF
