/*
 * Copyright (C) 2018-2021 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#pragma once

namespace NEO {

namespace SysCalls {

unsigned int getProcessId();

} // namespace SysCalls

} // namespace NEO
