/*
 * Copyright (C) 2018-2022 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#pragma once

namespace NEO {
enum class DriverModelType { UNKNOWN,
                             WDDM,
                             DRM };
};