/*
 * Copyright (C) 2019-2021 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#pragma once

#include <cstdint>

namespace GrfConfig {
constexpr uint32_t DefaultGrfNumber = 128u;
constexpr uint32_t LargeGrfNumber = 256u;
constexpr uint32_t NotApplicable = 0u;
} // namespace GrfConfig
