/*
 * Copyright (C) 2021-2022 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

namespace NEO {
unsigned int ultIterationMaxTime = 45;
const char *executionName = "OCLOC";
} // namespace NEO
