/*
 * Copyright (C) 2021 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#include "shared/source/device/device.h"

namespace NEO {

void Device::allocateRTDispatchGlobals(uint32_t maxBvhLevels) {
}

} // namespace NEO
