/*
 * Copyright (C) 2019-2022 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#include "shared/source/gen11/hw_info.h"

#include "opencl/source/command_queue/gpgpu_walker_bdw_and_later.inl"
#include "opencl/source/command_queue/hardware_interface_bdw_and_later.inl"

namespace NEO {

template class HardwareInterface<ICLFamily>;

template class GpgpuWalkerHelper<ICLFamily>;

template struct EnqueueOperation<ICLFamily>;

} // namespace NEO
