/*
 * Copyright (C) 2022 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#pragma once

#include <vector>

namespace NEO {
static const std::vector<unsigned short> PVC_XT_IDS{0x0BD5};
static const std::vector<unsigned short> PVC_XL_IDS{0x0BD0};
} // namespace NEO