/*
 * Copyright (C) 2020-2021 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#include "level_zero/tools/source/sysman/linux/firmware_util/firmware_util.h"
namespace L0 {
FirmwareUtil *FirmwareUtil::create(const std::string &pciBDF) {
    return nullptr;
}
} // namespace L0
