/*
 * Copyright (C) 2021-2022 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#include "shared/source/helpers/hw_info.h"

namespace NEO {

void overridePlatformName(std::string &name) {
}

} // namespace NEO
