/*
 * Copyright (C) 2021 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#include "shared/source/built_ins/built_ins.h"

namespace NEO {

const char *getUnknownBuiltinAsString(EBuiltInOps::Type builtin) {
    return "unknown";
}
} // namespace NEO
