/*
 * Copyright (C) 2019-2021 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#include "opencl/source/program/program.h"

namespace NEO {
void Program::applyAdditionalOptions(std::string &internalOptions) {
}
}; // namespace NEO
