/*
 * Copyright (C) 2019-2021 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#include "shared/source/command_stream/command_stream_receiver.h"

namespace NEO {

bool CommandStreamReceiver::createPerDssBackedBuffer(Device &device) {
    return true;
}
} // namespace NEO
