/*
 * Copyright (C) 2022 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#pragma once

namespace NEO {
enum IndirectHeapType {
    DYNAMIC_STATE = 0,
    INDIRECT_OBJECT,
    SURFACE_STATE,
    NUM_TYPES
};
} // namespace NEO
