/*
 * Copyright (C) 2018-2021 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#include "shared/source/helpers/ray_tracing_helper.h"

namespace NEO {

size_t RayTracingHelper::getRtGlobalsSize() {
    return 0;
}

} // namespace NEO
