/*
 * Copyright (C) 2021 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#include "shared/source/os_interface/linux/system_info.h"

namespace NEO {

void SystemInfo::checkSysInfoMismatch(HardwareInfo *hwInfo) {}
void SystemInfo::extendParseDeviceBlob(const uint32_t *data, uint32_t element) {}

} // namespace NEO
