/*
 * Copyright (C) 2020 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#pragma once

namespace NEO {
enum class MemoryCompressionState {
    Disabled = 0x0u,
    Enabled = 0x1u,
    NotApplicable = 0x2u
};
} // namespace NEO
