/*
 * Copyright (C) 2021 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#pragma once
#include "shared/source/xe_hpg_core/hw_cmds_base.h"

#ifdef SUPPORT_DG2
#include "hw_cmds_dg2.h"
#endif
