;
; Copyright (C) 2017-2021 Intel Corporation
;
; SPDX-License-Identifier: MIT
;

LIBRARY    "mock_opengl32"
EXPORTS
glGetString
wglGetProcAddress
glSetString
mockLoader
glGetStringiMock
wglSetSharedOCLContextStateINTELMock
wglGetCurrentContext
wglGetCurrentDC
glGetIntegerv
wglCreateContext
wglDeleteContext
wglShareLists
wglMakeCurrent
glSetString
glSetStringi
mockGLAcquireSharedBuffer
resetParam
getParam
loadBuffer
getBufferInfo
memParam
setGLSetSharedOCLContextStateReturnedValue
getGLSetSharedOCLContextStateReturnedValue
mockGLAcquireSharedRenderBuffer
mockGLReleaseSharedBuffer
mockGLReleaseSharedRenderBuffer
mockGLReleaseSharedTexture
mockGLAcquireSharedTexture
loadTexture
getTextureInfo
mockGLGetCurrentDisplay
setGlMockReturnedValues
getGlMockReturnedValues
mockGlRetainSync
mockGlReleaseSync
mockGlGetSynciv