/*
 * Copyright (C) 2021 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#include "shared/source/gen11/hw_cmds.h"
#include "shared/source/os_interface/hw_info_config.h"

namespace NEO {

static EnableProductHwInfoConfig<IGFX_ELKHARTLAKE> enableEHL;

} // namespace NEO
