/*
 * Copyright (C) 2021 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#include "shared/source/command_container/definitions/encode_surface_state_args_base.h"

namespace NEO {

template <typename GfxFamily>
void EncodeSurfaceState<GfxFamily>::appendBufferSurfaceState(EncodeSurfaceStateArgs &args) {
}

} // namespace NEO
